
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.


module Tomte
module Agent

  autoload :Common,       'tomte/agent/common'

  class Sync < Tomte::Agent::Common
    extend Tomte::Agent::Daemon

    attr_reader :management_endpoint
    attr_reader :endpoint

    def initialize(options = {})

      @management_bus = options[:management_bus] || :management
      @bus            = options[:bus]            || :service

      super
    end

    def run!(options = {})

      super
    end

    def shutdown!(options = {})
      super
    end

    private
    #######

    #
    # connects the agent to the tomte network
    #
    def connect_network!(bus = nil, &block)
      @bus = bus if bus

      @management_endpoint = Tomte::Protocol::Endpoint.new :sync => true, :bus => @management_bus
      @endpoint = Tomte::Protocol::Endpoint.new :sync => true, :bus => @bus

      [@endpoint, @management_endpoint].each { |ep| ep.connect! }
      block.call @endpoint if block_given?
    end

    #
    # disconnects the agent from the tomte network
    #
    def disconnect_network!(&block)
      block.call @endpoint if block_given?
      [@endpoint, @management_endpoint].each { |ep| ep.disconnect! if ep.try(:connected?) }
    end


    def start_agent(options = {})

      trap('TERM') { shutdown! }
      trap('INT') { shutdown! }

      run_agent
    end


    private
    #######

    def run
      log.debug "= agent #{self.class} has an empty run method"
    end

  end
end
end
