

# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte

  autoload :Worker,       'tomte/async/worker'

  module Agents
    module Framework

      def self.setup(options = { })

        Config.core[:argv] = options[:argv] if options[:argv]
        Tomte::Config.agents.merge options

        # best guess where the application is
        depth = RUBY_VERSION >= "1.9.2" ? 1 : 1

        application_path = options[:path] || caller(depth).first.split(':')[0]

        Config.core[:application] = options[:application] if options[:application]
        Config.core[:application] ||=  File.basename(application_path)
        Config.core[:application_root] ||= File.expand_path('..', File.dirname(application_path))
        Config.core[:application_lib] ||= File.join(Config.core[:application_root], 'lib')
        $:.unshift Config.core[:application_lib] if File.exist? Config.core[:application_lib]

        Config.agents.merge options
        Config.agents[:agents_dir] ||= File.expand_path(File.join(Config.core[:application_root], 'lib/tomte/agents'))

        agent_configuration_options

        if options[:interactive]
          Config.core[:interactive] = true
        else
          agent_options
        end
        async_agent_setup(options) if options[:async]

        if options[:file]
          require options[:file]
        elsif File.exist? Config.agents[:agents_dir]
          definition = options[:definition] || Config.core[:application]
          file = File.join(Config.agents[:agents_dir], "#{definition}.rb")

          begin
            require file
          rescue LoadError => e
            puts e.message.important!
            exit
          end
        end

      end

      def self.agent_configuration_options
        Tomte::Config.additional_options "Tomte Agent Configuration"  do |config, opts|
          opts.on("-c", "--configuration FILE", "Load the agent configuration from FILE") do |file|
            Tomte.config.add_config_files File.expand_path(file)
          end

          opts.on("--configurations-from GLOB", "Load the agent configuration from files matching GLOB") do |glob|
            Tomte::Config.add_config_files_matching File.expand_path(glob), {}
          end

        end
      end


      def self.agent_options
        Tomte::Config.additional_options "Tomte Agent Options"  do |config, opts|
          config[:agents] = {}

          opts.on("-d", "--daemonize", "Run as a daemon") do
            config[:agents][:daemonize] = true
          end

          opts.on("--no-daemonize", "Do not run as a daemon") do
            config[:agents][:daemonize] = false
          end

          if Tomte.config[:agents][:privilege_management]
            opts.on("--user USER", "Changes the user to USER before starting agent") do |user|
              config[:agents][:user] = user
            end

            opts.on("--group GROUP", "Changes the group to GROUP before starting agent") do |group|
              config[:agents][:group] = group
            end
          end

          opts.on("-C", "--chdir DIR", "Change to dir before starting agent") do |chdir|
            config[:agents][:chdir] = chdir
            Dir.chdir(chdir)
          end

          opts.on("--pidfile FILE", "Path to the pid file to create") do |file|
            config[:agents][:pid_file] = File.expand_path(file)
          end

          opts.on("--watchdog FILE", "Path to the watchdog socket") do |file|
            config[:agents][:watchdog_socket] = File.expand_path(file)
          end


          opts.on("--remote-debug", "Enable the remote debugger for workers") do
            config[:agents][:remote_debug] = true
          end

        end
      end


      def self.async_agent_setup(options = {})

        Tomte::Config.agents[:supervisor_interface] ||=  Tomte::Agents::SUPERVISOR_INTERFACE
        Tomte::Config.agents[:workers_dir] ||= File.expand_path(File.join(Config.core[:application_root], 'lib/tomte/workers'))
        Tomte::Config.agents[:tomte_workers_dir] ||= File.expand_path(File.join(Tomte::Agents::LIBRARY, 'tomte/async/workers'))


        if  File.exist? Tomte::Config.agents[:tomte_workers_dir]
          Dir[File.join(Tomte::Config.agents[:tomte_workers_dir], "*.rb")].each { |ext| require ext }
        end

        Tomte::Config.agents[:workers] = {}
        Tomte::Config.agents[:system_workers] = {}

        Tomte::Config.additional_options "Tomte Network Options"  do |config, opts|

          #opts.on("--supervisor-interface QUEUE", "Name of the Supervisor RPC interface [#{config[:agents][:supervisor_interface]}]") do |supervisor_interface|
          #  config[:agents][:supervisor_interface] = supervisor_interface
          #end

          opts.on("--supervised", "Wait for supervision before launching workers") do |supervised|
            config[:agents][:unsupervised] = !supervised
          end
        end
      end

      def self.preload_worker(klass, options, system = false)
        return if class_exists?(klass)

        if options[:file]
          file  = options[:file]
        else
          basedir = Tomte::Config.agents[(system ? :tomte_workers_dir : :workers_dir)]
          file = options[:definition] || ActiveSupport::Inflector.underscore(klass)
          file = File.join(basedir, file) unless file.include?(ActiveSupport::Inflector.underscore('Tomte::Workers'))
        end

        require(file)
      end

    end
  end
end

