
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

require 'eventmachine'
require 'tomte/async/actor/tuple'

module Tomte
  class Actor

    autoload :Filter,   'tomte/async/actor/filter'

    attr_accessor :process
    attr_accessor :filter

    def initialize
      _spawn_actor

      #initialize_events
      default_events
    end

    #
    # set a filter and a proc executed when matching
    #
    # append filter
    #
    # :blank => true reset the filter
    #
    def receive(args = {})
      raise ArgumentError, 'No filter block given.' unless block_given?

      # Build the filter
      if args[:blank]
        @filter = Filter.new(self)
      else
        @filter ||= Filter.new(self)
      end

      yield filter

      raise ArgumentError, 'Empty filter.' if filter.empty?
    end

    def notify(message)
      @process.notify(self, message)
    end

    alias :<< :notify

    private
    #######

    #
    # empty event handler for catching martians, and shutdowns
    #
    def default_events

      receive do |filter|

        filter.when(Object) do |message|
          martian message
        end
      end
    end

    #
    # called when receiving an message we don't know about
    #
    def martian(message)

    end


    #def self.events_declarations
    #  @events_declarations ||= []
    #end
    #
    #def self.receive_events(&block)
    #  events_declarations << block if block_given?
    #end
    #
    #
    #def initialize_events
    #  self.class.events_declarations.each do |decl|
    #    receive {|f| self.instance_exec(f, &decl) }
    #  end
    #end



    #
    # spawns a new EM spawned process
    #
    def _spawn_actor
      @process = EM.spawn do |obj, m|
        obj.send(:_receive, m)
      end
    end

    #
    # called when a message is in the mailbox
    #
    def _receive(message)
      raise ArgumentError, 'Received message with no filter in place.' if filter.nil?

      if(action = filter.match(message))
        action.call message
      end
   end


  end
end
