
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte
module Workers
  class Supervisor < Tomte::Worker

    class AgentAccessDenied < StandardError; end
    class InvalidRequest < StandardError; end

    class AgentInterface < Tomte::Worker::API::Server


      protected
      #########

      def initialize(supervisor, registry)
        super supervisor
        @supervisor = supervisor
        @registry = registry
      end



      def agent_sign_on(request)
        sanity_check request
        message = request.message

        if @registry.has_key?(message.header.uuid)
           raise AgentAccessDenied, "Duplicate UUID, #{message.header.uuid}"
        else
          _log.info "+ Registry: signing on agent: #{message.header.uuid} [#{message.header.identity}@#{message.header.host}]"
          _log.debug "+ #{message.pretty_inspect}"
          data = message.data
          check_data message.header, data
          @registry.learn(message.header.uuid, message.header, data)
          return Tomte::Message::Data.new(:result => :ok)
        end
      end

      def agent_sign_off(request)

        sanity_check request
        message = request.message
        _log.info "- Registry: signing off agent: #{message.header.uuid} [#{message.header.identity}@#{message.header.host}]"
        @registry.forget(message.header.uuid)

      end

      def agent_hello(request)

        sanity_check request
        message = request.message
        uuid = message.data.first

        _log.debug "+ hello from: #{uuid}"
        raise InvalidRequest, "UUID is mismatching" unless uuid == message.header.uuid
        @registry.touch(message.header.uuid)
      end

      def agent_update_workers(request)

        sanity_check request


      end



      private
      #######

      def sanity_check(request)
        if (i = @registry.check_message(request.message))
          raise InvalidRequest, "malformed request: #{i}"
        end
      end

      #
      # check if mandatory elements in the header and in the request data are the same
      #
      def check_data(header, request_data)
        AgentRegistry::MANDATORY_IE.each do |i|
          raise InvalidRequest, "local data is mismatching" if !header[i].nil? && (header[i] != request_data.local[i])
        end
      end

    end
  end
end
end



