
# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.


module Tomte
module Workers
  class Supervisor < Tomte::Worker
    class API < Tomte::Worker::API::Server

      def initialize(supervisor, registry)
        super supervisor
        @supervisor = supervisor
        @registry = registry
      end

      def status
        _log.debug "+ supervisor api: got status request "
        status = "Tomte Supervisor\n" + Tomte::Message.local_info.to_human
        return status
      end

      def delete(what, id)
        _get(what).delete id
      end

      def clear(what)

        _get(what).clear
      end

      def flush(what)

        _get(what).flush
      end


      def show(what = :registry)

        _log.debug "+ supervisor api: got show request #{what}"
        if _human_readable?
          _get(what).show
        else
          _get(what).to_h
        end
      end


      def debug_count_objects
        ObjectSpace.count_objects
      end

      def debug_object_classes
        counts = {}
        ObjectSpace.each_object {|o|  counts[o.class.name] ||= 0; counts[o.class.name] +=1 }
        counts
      end

      def debug_gc
        ObjectSpace.garbage_collect
      end

      private
      #######

      def _get(what)
         case what.to_sym
         when :registry
          @registry
        else
          raise ArgumentError, "Don't know how to get #{what}"
        end

      end

    end
  end
end
end



