# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte::Network
  module API

    #
    # the blankslate class that you extend to implement an API
    #
    class BasicServer

      SAFE_METHODS = [:methods, :object_id, :class, :kind_of?, :is_a?, :respond_to?, :respond_to_missing?].freeze
      alias :__instance_eval :instance_eval

      ## this is similar to a BlankSlate, but with a different set of safe methods
      instance_methods.each { |m|  undef_method m unless ( SAFE_METHODS.include?(m.to_sym) || m.to_s =~ /^__/) }

      attr_reader :_log

      def initialize(worker, params = {})
        @api_name   = params.fetch(:name, 'unnamed-api')

        @target = params[:target]
        @worker = worker
        @_log = @worker.log
      end

      def api_help
        "== HELP =="
      end

      def ping
        :pong
      end

      def methods
        super.select { |m| !(m.to_s =~ /^_/) }.sort
      end

      def banner
        {:api => @api_name.to_s, :class => self.class.name, :versions => _banner_versions, :simple => true}
      end

      def interface
      end

      alias :api_methods :methods

      private
      #######

      def _banner_versions

        { :engine    => RUBY_ENGINE,
          :runtime   => RUBY_VERSION,
          :framework => Tomte::VERSION,
          :protocol  => Tomte::Protocol::VERSION,
          :agents    => Tomte::Agents::VERSION,
          :api       => _api_version }
      end

      def _api_version
        "0.0.0"
      end

      def _human_readable?
        false
      end
    end
  end
end
