# Tomte, a framework for distributed agents.
# Copyright (c) 2010-2012, Lele Forzani, windmillmedia. All Rights Reserved.
#
# Author:: Lele Forzani <lele@windmill.it>
# License:: You can redistribute it and/or modify it under the terms of the LICENSE file.

module Tomte
  module Network
    module Bus

      DEFAULT_ENVELOPER = Tomte::Protocol::Enveloper::Generic
      EP = Struct.new(:ep, :ready, :closed)

      extend ActiveSupport::Concern

      included do
        extend ActiveSupport::DescendantsTracker

        class_attribute :buses
        self.buses ||= {}

        class << self
          prepend Inheritance
        end
      end

      module Inheritance
        def inherited(subclass)
          super(subclass)
          subclass.buses = self.buses.dup
        end
      end

      module ClassMethods

        #
        # declares a bus for this worker
        #
        def bus(name, options = {}, &block)
          buses[name] = {:options => options, :block => block}
        end
      end


      #
      # returns the endpoint with name
      #
      def endpoint(name)
        endpoints[name].ep
      end

      #
      # returns the hash of all endpoints for this worker
      #
      def endpoints
        @endpoints ||= {}
      end

      private
      #######


      def close_endpoints
        all_endpoints_closed?

        endpoints.each do |name, endp|

          endp.ep.disconnect! { endpoints[name].closed = true; all_endpoints_closed? }
        end
      end

      def initialize_endpoints(&after_all_done)

        append_callback :all_endpoints_closed, &after_all_done
        append_callback :before_shutdown, &method(:close_endpoints)

        buses.each do |name, params|
          new_endpoint(name, params[:options].dup, &params[:block])
        end

        all_endpoints_ready?
      end

      def new_endpoint(name, options = {}, &block)

        options[:enveloper] ||= DEFAULT_ENVELOPER.new

        endpoints[name] =  EP.new(Tomte::Protocol::Endpoint.new({ :bus => name,
                                                                  :async => true}.merge(options)))
        endpoints[name].ep.connect! do |bus|

          endpoints[name].ready = true

          # call the block in the context of the worker
          instance_exec(bus, &block) if block

          all_endpoints_ready?
        end
      end

      def all_endpoints_ready?
        #state = endpoints.inject(true){ |state, (name, ep)| ep.ready }
        state = endpoints.all? { | name, ep | ep.ready }
        exec_callback_once :all_endpoints_ready if state
        state
      end

      def all_endpoints_closed?
        #state = endpoints.inject(true){ |state, (name, ep)| ep.closed }
        state = endpoints.all? { | name, ep | ep.closed }
        exec_callback_once :all_endpoints_closed if state
        state
      end


    end
  end
end